#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/10 4:03 PM
# @File    : empirical_func.py
# @Software: PyCharm


import itertools
import numpy as np
import pandas as pd
from datetime import datetime
from datetime import timedelta


def empirical_CDF(array, x):
    return sum(array <= x) / array.size


def cvm_statistic(array1, array2):
    k = array1.size
    if not array2.size == k:
        print('array1 and array2 have different sizes!')
        return None
    denom = 0
    f1 = lambda x: empirical_CDF(array1, x)
    f2 = lambda x: empirical_CDF(array2, x)
    for x in np.concatenate((array1, array2)):
        denom += (f1(x) - f2(x)) ** 2
    return denom / (2 * k)


def critical_value(array1, array2, alphas):
    k = array1.size
    array = np.concatenate((array1, array2))
    t_list = []
    all_pi = []
    if k < 5:
        all_pi = list(itertools.permutations(array))
        m = len(all_pi)
    if k >= 5:
        m = 100000  # number of permutations using in computing CV
        for i in range(m):
            all_pi.append(np.random.permutation(array))
    for pi in all_pi:
        pi = np.array(pi)
        pi_1 = pi[:k]
        pi_2 = pi[k:]
        t_list.append(cvm_statistic(pi_1, pi_2))
    t_list.sort()
    t_criticals = []
    for alpha in alphas:
        k_critical = int(m * (1 - alpha)) + 1
        t_criticals.append(t_list[k_critical - 1])
    return t_criticals


def permutation_test(array1, array2, alphas=None):
    if alphas == None:
        alphas = [0.1, 0.05, 0.01]
    t = cvm_statistic(array1, array2)
    t_criticals = critical_value(array1, array2, alphas)
    result = []
    for i in range(len(t_criticals)):
        result.append(t < t_criticals[i])
    return result


def transform_data(file):
    data = pd.read_csv(file)
    data = data.rename(columns={'Last': 'Close'})
    data['Return'] = np.log(data['Close']) - np.log(data['Open'])
    data['Abs_Return'] = np.abs(data['Return'])
    data['Amihud'] = data['Abs_Return'] / data['Money']
    for i in range(len(data.Time)):
        data.loc[i, 'Time'] = datetime.strptime(data.Time[i], '%m/%d/%Y %H:%M')
    return data


def get_arrays(data, time, keyword, discard=5, kn=5):
    time = datetime.strptime(time, '%Y-%m-%d %H:%M')
    li1 = []
    li2 = []
    for i in range(kn):
        i = discard + i + 1
        dt1 = timedelta(minutes=i)
        dt2 = timedelta(minutes=i + 1)
        li1.append(data.loc[data.Time == time - dt1, keyword].values[0])
        li2.append(data.loc[data.Time == time + dt2, keyword].values[0])
    array1 = np.array(li1[::-1])
    array2 = np.array(li2)
    return array1, array2


def run_test(file, time, keyword, discard=5, kn=5, alphas=None):
    if alphas is None:
        alphas = [0.1, 0.05, 0.01]
    data = transform_data(file)
    array1, array2 = get_arrays(data, time, keyword, discard, kn)
    return permutation_test(array1, array2, alphas)


def get_times(data, time, discard=5, kn=5):
    time = datetime.strptime(time, '%Y-%m-%d %H:%M')
    li1 = []
    li2 = []
    for i in range(kn):
        i = discard + i + 1
        dt1 = timedelta(minutes=i)
        dt2 = timedelta(minutes=i + 1)
        li1.append(data.loc[data.Time == time - dt1, 'Time'].values[0])
        li2.append(data.loc[data.Time == time + dt2, 'Time'].values[0])
    return li1[::-1], li2


def transform_spread(file):
    data = pd.read_csv(file)
    data['TIME_M'] = data['TIME_M'].map(lambda x: x[0:x.find(':') + 3])
    data['DATE'] = data['DATE'].map(lambda x: str(x))
    data['DATE'] = data['DATE'].map(lambda x: x[0:4] + '-' + x[4:6] + '-' + x[6:8])
    data['Time'] = data['DATE'] + ' ' + data['TIME_M']
    data['Spread'] = data['BEST_ASK'] - data['BEST_BID']
    data.drop(['DATE', 'TIME_M', 'SYM_ROOT', 'BEST_BID', 'BEST_ASK'], axis=1, inplace=True)
    data = data.groupby(['Time']).mean()
    data = data.reset_index()
    data['Time'] = data['Time'].map(lambda x: datetime.strptime(x, '%Y-%m-%d %H:%M'))
    return data


def run_test_spread(file, time, keyword, discard=5, kn=5, alphas=None):
    if alphas == None:
        alphas = [0.1, 0.05, 0.01]
    data = transform_spread(file)
    array1, array2 = get_arrays(data, time, keyword, discard, kn)
    return permutation_test(array1, array2, alphas)
