function simul = ptvj_dgp(beta,J,kn,m_frequency,trunc)
% We simulate at 1 second frequency. Then aggregate the 1-sec returns into
% m_frequency-second returns. kn is the (largest) one-sided window size. This means
% that if we sample (2*kn + 1) m_frequency-second returns, we should have enough.
% J is the jump size of volatility. beta is the activity of stable process. 
% trunc is used to truncate the stable shocks.

deltan = 1/390/60; % 1 second Euler scheme
rho = -0.7; % leverage effect
nobs = m_frequency * (2*kn+1); % number of 1-sec returns

dB1 = randn(nobs,1) * sqrt(deltan); % Brownian shock in vol factor 1
dB2 = randn(nobs,1) * sqrt(deltan); % Brownian shock in vol factor 2
dL  = stabletrunc(beta,trunc,nobs) * deltan^(1/beta);
    % Levy shock in price, with truncated beta-stable distribution

% Add one jump at the end of pre-event window
voljump = zeros(nobs,1);
voljump(m_frequency*kn+1) = J;

% Initializing
V1 = zeros(nobs,1); V1(1) = 0.5;
V2 = zeros(nobs,1); V2(1) = 0.5;

% Diagonostic flags
err_count = 0; % Unsuccessful trials (i.e., negative variance)
finish_flag = 0; % Successful simulation

% Simulation via Euler
while ~finish_flag
    err_flag = 0;
    for i = 2:nobs
        V1(i) = V1(i-1) + 0.0116 * (0.5 - V1(i-1)) * deltan ...
                        + 0.1023 * sqrt(V1(i-1)) * (rho * dL(i-1) + sqrt(1-rho^2) * dB1(i-1)) + voljump(i);
        V2(i) = V2(i-1) + 0.6930 * (0.5 - V2(i-1)) * deltan ...
                        + 0.7909 * sqrt(V2(i-1)) * (rho * dL(i-1) + sqrt(1-rho^2) * dB2(i-1)) + voljump(i); 
        if V1(i) < 0 || V2(i) < 0
            % if get negative variance, flag err, redraw
            err_flag = 1;
            err_count = err_count + 1;
            dB1 = randn(nobs,1) * sqrt(deltan);
            dB2 = randn(nobs,1) * sqrt(deltan);
            dL  = stabletrunc(beta,trunc,nobs) * deltan^(1/beta);            
            break;
        end
    end
    if err_count > 100
        error('DGP does not really work!') % Give up
    end
    if ~err_flag
        % if loop finishes without err, finish, otherwise rerun
        finish_flag = 1;
    end
end

xall_a = sqrt(2 * V1) .* dL;  % all returns in model a
xall_b = sqrt(V1 + V2) .* dL; % all returns in model b
xa = sum(reshape(xall_a,m_frequency,2*kn+1),1)'; % resampled returns in model a
xb = sum(reshape(xall_b,m_frequency,2*kn+1),1)'; % resampled returns in model b

% two samples of returns in model a
simul.xa1 = xa(1:kn);
simul.xa2 = xa(kn+2:end);

% two samples of returns in model b
simul.xb1 = xb(1:kn);
simul.xb2 = xb(kn+2:end);

% Other things
simul.kn = kn;
simul.V1 = V1;
simul.V2 = V2;

% helper function
function x = stabletrunc(beta,trunc,n)
% simulate truncated beta-stable random variables
x = random('Stable',beta,0,1/sqrt(2),0,n,1);
    % simulate stable variable, setting the scale to 1/sqrt(2) to recover
    % Brownian motion when beta = 2
while sum(abs(x) > trunc) > 0
    % redraw the variables greater than truncation level
    tmpindex = (abs(x)>trunc);
    tmpn = sum(tmpindex);
    x(tmpindex) = random('Stable',beta,0,1/sqrt(2),0,tmpn,1);
end